@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,16 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2014
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2015
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 import rucio.api.permission
 import rucio.common.exception
@@ -32,22 +22,26 @@ from rucio.common.types import InternalAccount
 from rucio.core import account_limit as account_limit_core
 from rucio.core.account import account_exists
 from rucio.core.rse import get_rse_id, get_rse_name
+from rucio.db.sqla.session import read_session, transactional_session
 
 
-def get_rse_account_usage(rse, vo='def'):
+@read_session
+def get_rse_account_usage(rse, vo='def', session=None):
     """
     Returns the account limit and usage for all for all accounts on a RSE.
 
     :param rse:      The RSE name.
     :param vo:       The VO to act on.
+    :param session:  The database session in use.
     :return:         List of dictionnaries.
     """
-    rse_id = get_rse_id(rse=rse, vo=vo)
+    rse_id = get_rse_id(rse=rse, vo=vo, session=session)
 
-    return [api_update_return_dict(d) for d in account_limit_core.get_rse_account_usage(rse_id=rse_id)]
+    return [api_update_return_dict(d, session=session) for d in account_limit_core.get_rse_account_usage(rse_id=rse_id, session=session)]
 
 
-def get_local_account_limits(account, vo='def'):
+@read_session
+def get_local_account_limits(account, vo='def', session=None):
     """
     Lists the limitation names/values for the specified account name.
 
@@ -55,6 +49,7 @@ def get_local_account_limits(account, vo='def'):
 
     :param account:     The account name.
     :param vo:          The VO to act on.
+    :param session:     The database session in use.
 
     :returns: The account limits.
     """
@@ -62,12 +57,13 @@ def get_local_account_limits(account, vo='def'):
     account = InternalAccount(account, vo=vo)
 
     rse_instead_id = {}
-    for elem in account_limit_core.get_local_account_limits(account=account).items():
-        rse_instead_id[get_rse_name(rse_id=elem[0])] = elem[1]
+    for elem in account_limit_core.get_local_account_limits(account=account, session=session).items():
+        rse_instead_id[get_rse_name(rse_id=elem[0], session=session)] = elem[1]
     return rse_instead_id
 
 
-def get_local_account_limit(account, rse, vo='def'):
+@read_session
+def get_local_account_limit(account, rse, vo='def', session=None):
     """
     Lists the limitation names/values for the specified account name and rse name.
 
@@ -76,17 +72,19 @@ def get_local_account_limit(account, rse, vo='def'):
     :param account:     The account name.
     :param rse:         The rse name.
     :param vo:          The VO to act on.
+    :param session:     The database session in use.
 
     :returns: The account limit.
     """
 
     account = InternalAccount(account, vo=vo)
 
-    rse_id = get_rse_id(rse=rse, vo=vo)
-    return {rse: account_limit_core.get_local_account_limit(account=account, rse_id=rse_id)}
+    rse_id = get_rse_id(rse=rse, vo=vo, session=session)
+    return {rse: account_limit_core.get_local_account_limit(account=account, rse_id=rse_id, session=session)}
 
 
-def get_global_account_limits(account, vo='def'):
+@read_session
+def get_global_account_limits(account, vo='def', session=None):
     """
     Lists the limitation names/values for the specified account name.
 
@@ -94,6 +92,7 @@ def get_global_account_limits(account, vo='def'):
 
     :param account:     The account name.
     :param vo:          The VO to act on.
+    :param session:     The database session in use.
 
     :returns: The account limits.
     """
@@ -102,10 +101,11 @@ def get_global_account_limits(account, vo='def'):
     else:
         account = InternalAccount('*', vo=vo)
 
-    return account_limit_core.get_global_account_limits(account=account)
+    return account_limit_core.get_global_account_limits(account=account, session=session)
 
 
-def get_global_account_limit(account, rse_expression, vo='def'):
+@read_session
+def get_global_account_limit(account, rse_expression, vo='def', session=None):
     """
     Lists the limitation names/values for the specified account name and rse expression.
 
@@ -114,16 +114,18 @@ def get_global_account_limit(account, rse_expression, vo='def'):
     :param account:         The account name.
     :param rse_expression:  The rse expression.
     :param vo:              The VO to act on.
+    :param session:         The database session in use.
 
     :returns: The account limit.
     """
 
     account = InternalAccount(account, vo=vo)
 
-    return {rse_expression: account_limit_core.get_global_account_limit(account=account, rse_expression=rse_expression)}
+    return {rse_expression: account_limit_core.get_global_account_limit(account=account, rse_expression=rse_expression, session=session)}
 
 
-def set_local_account_limit(account, rse, bytes_, issuer, vo='def'):
+@transactional_session
+def set_local_account_limit(account, rse, bytes_, issuer, vo='def', session=None):
     """
     Set an account limit..
 
@@ -132,22 +134,24 @@ def set_local_account_limit(account, rse, bytes_, issuer, vo='def'):
     :param bytes_:   The limit in bytes.
     :param issuer:  The issuer account_core.
     :param vo:      The VO to act on.
+    :param session: The database session in use.
     """
-    rse_id = get_rse_id(rse=rse, vo=vo)
+    rse_id = get_rse_id(rse=rse, vo=vo, session=session)
 
     kwargs = {'account': account, 'rse': rse, 'rse_id': rse_id, 'bytes': bytes_}
-    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='set_local_account_limit', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='set_local_account_limit', kwargs=kwargs, session=session):
         raise rucio.common.exception.AccessDenied('Account %s can not set account limits.' % (issuer))
 
     account = InternalAccount(account, vo=vo)
 
-    if not account_exists(account=account):
+    if not account_exists(account=account, session=session):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
 
-    account_limit_core.set_local_account_limit(account=account, rse_id=rse_id, bytes_=bytes_)
+    account_limit_core.set_local_account_limit(account=account, rse_id=rse_id, bytes_=bytes_, session=session)
 
 
-def set_global_account_limit(account, rse_expression, bytes_, issuer, vo='def'):
+@transactional_session
+def set_global_account_limit(account, rse_expression, bytes_, issuer, vo='def', session=None):
     """
     Set a global account limit.
 
@@ -156,21 +160,23 @@ def set_global_account_limit(account, rse_expression, bytes_, issuer, vo='def'):
     :param bytes_:           The limit in bytes.
     :param issuer:          The issuer account_core.
     :param vo:              The VO to act on.
+    :param session:         The database session in use.
     """
 
     kwargs = {'account': account, 'rse_expression': rse_expression, 'bytes': bytes_}
-    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='set_global_account_limit', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='set_global_account_limit', kwargs=kwargs, session=session):
         raise rucio.common.exception.AccessDenied('Account %s can not set account limits.' % (issuer))
 
     account = InternalAccount(account, vo=vo)
 
-    if not account_exists(account=account):
+    if not account_exists(account=account, session=session):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
 
-    account_limit_core.set_global_account_limit(account=account, rse_expression=rse_expression, bytes_=bytes_)
+    account_limit_core.set_global_account_limit(account=account, rse_expression=rse_expression, bytes_=bytes_, session=session)
 
 
-def delete_local_account_limit(account, rse, issuer, vo='def'):
+@transactional_session
+def delete_local_account_limit(account, rse, issuer, vo='def', session=None):
     """
     Delete an account limit..
 
@@ -178,24 +184,26 @@ def delete_local_account_limit(account, rse, issuer, vo='def'):
     :param rse:     The rse name.
     :param issuer:  The issuer account_core.
     :param vo:      The VO to act on.
+    :param session: The database session in use.
 
     :returns: True if successful; False otherwise.
     """
 
-    rse_id = get_rse_id(rse=rse, vo=vo)
+    rse_id = get_rse_id(rse=rse, vo=vo, session=session)
     kwargs = {'account': account, 'rse': rse, 'rse_id': rse_id}
-    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='delete_local_account_limit', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='delete_local_account_limit', kwargs=kwargs, session=session):
         raise rucio.common.exception.AccessDenied('Account %s can not delete account limits.' % (issuer))
 
     account = InternalAccount(account, vo=vo)
 
-    if not account_exists(account=account):
+    if not account_exists(account=account, session=session):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
 
-    return account_limit_core.delete_local_account_limit(account=account, rse_id=rse_id)
+    return account_limit_core.delete_local_account_limit(account=account, rse_id=rse_id, session=session)
 
 
-def delete_global_account_limit(account, rse_expression, issuer, vo='def'):
+@transactional_session
+def delete_global_account_limit(account, rse_expression, issuer, vo='def', session=None):
     """
     Delete a global account limit..
 
@@ -203,23 +211,25 @@ def delete_global_account_limit(account, rse_expression, issuer, vo='def'):
     :param rse_expression: The rse expression.
     :param issuer:         The issuer account_core.
     :param vo:             The VO to act on.
+    :param session:        The database session in use.
 
     :returns: True if successful; False otherwise.
     """
 
     kwargs = {'account': account, 'rse_expression': rse_expression}
-    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='delete_global_account_limit', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='delete_global_account_limit', kwargs=kwargs, session=session):
         raise rucio.common.exception.AccessDenied('Account %s can not delete global account limits.' % (issuer))
 
     account = InternalAccount(account, vo=vo)
 
-    if not account_exists(account=account):
+    if not account_exists(account=account, session=session):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
 
-    return account_limit_core.delete_global_account_limit(account=account, rse_expression=rse_expression)
+    return account_limit_core.delete_global_account_limit(account=account, rse_expression=rse_expression, session=session)
 
 
-def get_local_account_usage(account, rse, issuer, vo='def'):
+@read_session
+def get_local_account_usage(account, rse, issuer, vo='def', session=None):
     """
     Get the account usage and connect it with (if available) the account limits of the account.
 
@@ -227,6 +237,7 @@ def get_local_account_usage(account, rse, issuer, vo='def'):
     :param rse:      The rse to read (If none, get all).
     :param issuer:   The issuer account.
     :param vo:       The VO to act on.
+    :param session:  The database session in use.
 
     :returns:        List of dicts {'rse_id', 'rse', 'bytes', 'files', 'bytes_limit', 'bytes_remaining'}
     """
@@ -234,20 +245,21 @@ def get_local_account_usage(account, rse, issuer, vo='def'):
     rse_id = None
 
     if rse:
-        rse_id = get_rse_id(rse=rse, vo=vo)
+        rse_id = get_rse_id(rse=rse, vo=vo, session=session)
     kwargs = {'account': account, 'rse': rse, 'rse_id': rse_id}
-    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='get_local_account_usage', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='get_local_account_usage', kwargs=kwargs, session=session):
         raise rucio.common.exception.AccessDenied('Account %s can not list account usage.' % (issuer))
 
     account = InternalAccount(account, vo=vo)
 
-    if not account_exists(account=account):
+    if not account_exists(account=account, session=session):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
 
-    return [api_update_return_dict(d) for d in account_limit_core.get_local_account_usage(account=account, rse_id=rse_id)]
+    return [api_update_return_dict(d, session=session) for d in account_limit_core.get_local_account_usage(account=account, rse_id=rse_id, session=session)]
 
 
-def get_global_account_usage(account, rse_expression, issuer, vo='def'):
+@read_session
+def get_global_account_usage(account, rse_expression, issuer, vo='def', session=None):
     """
     Get the account usage and connect it with (if available) the account limits of the account.
 
@@ -255,17 +267,18 @@ def get_global_account_usage(account, rse_expression, issuer, vo='def'):
     :param rse_expression:  The rse expression to read (If none, get all).
     :param issuer:          The issuer account.
     :param vo:              The VO to act on.
+    :param session:         The database session in use.
 
     :returns:               List of dicts {'rse_id', 'rse', 'bytes', 'files', 'bytes_limit', 'bytes_remaining'}
     """
 
     kwargs = {'account': account, 'rse_expression': rse_expression}
-    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='get_global_account_usage', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='get_global_account_usage', kwargs=kwargs, session=session):
         raise rucio.common.exception.AccessDenied('Account %s can not list global account usage.' % (issuer))
 
     account = InternalAccount(account, vo=vo)
 
-    if not account_exists(account=account):
+    if not account_exists(account=account, session=session):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
 
-    return [api_update_return_dict(d) for d in account_limit_core.get_global_account_usage(account=account, rse_expression=rse_expression)]
+    return [api_update_return_dict(d, session=session) for d in account_limit_core.get_global_account_usage(account=account, rse_expression=rse_expression, session=session)]
