@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,32 +12,9 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2017
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2021
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2013
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2020
-# - Yun-Pin Sun <winter0128@gmail.com>, 2013
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2013
-# - Martin Barisits <martin.barisits@cern.ch>, 2014-2020
-# - asket <asket.agarwal96@gmail.com>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Eric Vaandering <ewv@fnal.gov>, 2020
-# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2020
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2020
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Rob Barnsley <robbarnsley@users.noreply.github.com>, 2021-2022
-# - Radu Carpa <radu.carpa@cern.ch>, 2022
-
-from __future__ import print_function
 
 from copy import deepcopy
+from typing import TYPE_CHECKING
 
 import rucio.api.permission
 from rucio.common.constants import RESERVED_KEYS
@@ -50,6 +27,10 @@ from rucio.core.rse import get_rse_id
 from rucio.db.sqla.constants import DIDType
 from rucio.db.sqla.session import read_session, stream_session, transactional_session
 
+if TYPE_CHECKING:
+    from typing import Any, Dict, Optional
+    from sqlalchemy.orm import Session
+
 
 @stream_session
 def list_dids(scope, filters, did_type='collection', ignore_case=False, limit=None, offset=None, long=False, recursive=False, vo='def', session=None):
@@ -423,13 +404,15 @@ def scope_list(scope, name=None, recursive=False, vo='def', session=None):
 
 
 @read_session
-def get_did(scope, name, dynamic=False, vo='def', session=None):
+def get_did(scope: str, name: str, dynamic_depth: "Optional[DIDType]" = None, vo: str = 'def', session: "Optional[Session]" = None) -> "Dict[str, Any]":
     """
     Retrieve a single data did.
 
     :param scope: The scope name.
     :param name: The data identifier name.
-    :param dynamic:  Dynamically resolve the bytes and length of the did
+    :param dynamic_depth: the DID type to use as source for estimation of this DIDs length/bytes.
+    If set to None, or to a value which doesn't make sense (ex: requesting depth = CONTAINER for a did of type DATASET)
+    will not compute the size dynamically.
     :param vo: The VO to act on.
     :return did: Dictionary containing {'name', 'scope', 'type'}, Exception otherwise
     :param session: The database session in use.
@@ -437,7 +420,7 @@ def get_did(scope, name, dynamic=False, vo='def', session=None):
 
     scope = InternalScope(scope, vo=vo)
 
-    d = did.get_did(scope=scope, name=name, dynamic=dynamic, session=session)
+    d = did.get_did(scope=scope, name=name, dynamic_depth=dynamic_depth, session=session)
     return api_update_return_dict(d, session=session)
 
 
