@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2020 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,16 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2020
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2015
-# - Tomáš Kouba <tomas.kouba@cern.ch>, 2014
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2014
-# - Martin Barisits <martin.barisits@cern.ch>, 2017
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
 
 """
 Interface for identity abstraction layer
@@ -32,9 +22,11 @@ from rucio.common import exception
 from rucio.common.types import InternalAccount
 from rucio.core import identity
 from rucio.db.sqla.constants import IdentityType
+from rucio.db.sqla.session import read_session, transactional_session
 
 
-def add_identity(identity_key, id_type, email, password=None):
+@transactional_session
+def add_identity(identity_key, id_type, email, password=None, session=None):
     """
     Creates a user identity.
 
@@ -42,27 +34,31 @@ def add_identity(identity_key, id_type, email, password=None):
     :param id_type: The type of the authentication (x509, gss, userpass, ssh, saml)
     :param email: The Email address associated with the identity.
     :param password: If type==userpass, this sets the password.
+    :param session: The database session in use.
     """
-    return identity.add_identity(identity_key, IdentityType[id_type.upper()], email, password=password)
+    return identity.add_identity(identity_key, IdentityType[id_type.upper()], email, password=password, session=session)
 
 
-def del_identity(identity_key, id_type, issuer, vo='def'):
+@transactional_session
+def del_identity(identity_key, id_type, issuer, vo='def', session=None):
     """
     Deletes a user identity.
     :param identity_key: The identity key name. For example x509 DN, or a username.
     :param id_type: The type of the authentication (x509, gss, userpass, ssh, saml).
     :param issuer: The issuer account.
     :param vo: the VO of the issuer.
+    :param session: The database session in use.
     """
     id_type = IdentityType[id_type.upper()]
-    kwargs = {'accounts': identity.list_accounts_for_identity(identity_key, id_type)}
-    if not permission.has_permission(issuer=issuer, vo=vo, action='del_identity', kwargs=kwargs):
+    kwargs = {'accounts': identity.list_accounts_for_identity(identity_key, id_type, session=session)}
+    if not permission.has_permission(issuer=issuer, vo=vo, action='del_identity', kwargs=kwargs, session=session):
         raise exception.AccessDenied('Account %s can not delete identity' % (issuer))
 
-    return identity.del_identity(identity_key, id_type)
+    return identity.del_identity(identity_key, id_type, session=session)
 
 
-def add_account_identity(identity_key, id_type, account, email, issuer, default=False, password=None, vo='def'):
+@transactional_session
+def add_account_identity(identity_key, id_type, account, email, issuer, default=False, password=None, vo='def', session=None):
     """
     Adds a membership association between identity and account.
 
@@ -74,17 +70,20 @@ def add_account_identity(identity_key, id_type, account, email, issuer, default=
     :param default: If True, the account should be used by default with the provided identity.
     :param password: Password if id_type is userpass.
     :param vo: the VO to act on.
+    :param session: The database session in use.
     """
     kwargs = {'identity': identity_key, 'type': id_type, 'account': account}
-    if not permission.has_permission(issuer=issuer, vo=vo, action='add_account_identity', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='add_account_identity', kwargs=kwargs, session=session):
         raise exception.AccessDenied('Account %s can not add account identity' % (issuer))
 
     account = InternalAccount(account, vo=vo)
 
-    return identity.add_account_identity(identity=identity_key, type_=IdentityType[id_type.upper()], default=default, email=email, account=account, password=password)
+    return identity.add_account_identity(identity=identity_key, type_=IdentityType[id_type.upper()], default=default,
+                                         email=email, account=account, password=password, session=session)
 
 
-def del_account_identity(identity_key, id_type, account, issuer, vo='def'):
+@transactional_session
+def del_account_identity(identity_key, id_type, account, issuer, vo='def', session=None):
     """
     Removes a membership association between identity and account.
 
@@ -93,44 +92,51 @@ def del_account_identity(identity_key, id_type, account, issuer, vo='def'):
     :param account: The account name.
     :param issuer: The issuer account.
     :param vo: the VO to act on.
+    :param session: The database session in use.
     """
     kwargs = {'account': account}
-    if not permission.has_permission(issuer=issuer, vo=vo, action='del_account_identity', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='del_account_identity', kwargs=kwargs, session=session):
         raise exception.AccessDenied('Account %s can not delete account identity' % (issuer))
 
     account = InternalAccount(account, vo=vo)
 
-    return identity.del_account_identity(identity_key, IdentityType[id_type.upper()], account)
+    return identity.del_account_identity(identity_key, IdentityType[id_type.upper()], account, session=session)
 
 
-def list_identities(**kwargs):
+@read_session
+def list_identities(session=None, **kwargs):
     """
     Returns a list of all enabled identities.
 
+    :param session: The database session in use.
     returns: A list of all enabled identities.
     """
-    return identity.list_identities(**kwargs)
+    return identity.list_identities(session=session, **kwargs)
 
 
-def get_default_account(identity_key, id_type):
+@read_session
+def get_default_account(identity_key, id_type, session=None):
     """
     Returns the default account for this identity.
 
     :param identity_key: The identity key name. For example x509 DN, or a username.
     :param id_type: The type of the authentication (x509, gss, userpass, ssh, saml).
+    :param session: The database session in use.
     """
-    account = identity.get_default_account(identity_key, IdentityType[id_type.upper()])
+    account = identity.get_default_account(identity_key, IdentityType[id_type.upper()], session=session)
     return account.external
 
 
-def list_accounts_for_identity(identity_key, id_type):
+@read_session
+def list_accounts_for_identity(identity_key, id_type, session=None):
     """
     Returns a list of all accounts for an identity.
 
     :param identity: The identity key name. For example x509 DN, or a username.
     :param id_type: The type of the authentication (x509, gss, userpass, ssh, saml).
+    :param session: The database session in use.
 
     returns: A list of all accounts for the identity.
     """
-    accounts = identity.list_accounts_for_identity(identity_key, IdentityType[id_type.upper()])
+    accounts = identity.list_accounts_for_identity(identity_key, IdentityType[id_type.upper()], session=session)
     return [account.external for account in accounts]
