@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,42 +12,11 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2020
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
-# - Yun-Pin Sun <winter0128@gmail.com>, 2013
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2021
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2020
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2015
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2021
-# - Tobias Wegner <twegner@cern.ch>, 2017
-# - Brian Bockelman <bbockelm@cse.unl.edu>, 2017-2018
-# - Robert Illingworth <illingwo@fnal.gov>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
-# - Tomas Javurek <tomas.javurek@cern.ch>, 2019-2020
-# - Brandon White <bjwhite@fnal.gov>, 2019
-# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
-# - Eric Vaandering <ewv@fnal.gov>, 2019
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Paul Millar <paul.millar@desy.de>, 2021
-# - martynia <martynia@users.noreply.github.com>, 2021
-# - Igor Mandrichenko <ivm@fnal.gov>, 2021
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2022
 
 '''
  Client class for callers of the Rucio system
 '''
 
-from __future__ import print_function
-
 import errno
 import getpass
 import os
@@ -62,8 +31,8 @@ from dogpile.cache import make_region
 from requests import Session, Response
 from requests.exceptions import ConnectionError
 from requests.status_codes import codes
-from six.moves.configparser import NoOptionError, NoSectionError
-from six.moves.urllib.parse import urlparse
+from configparser import NoOptionError, NoSectionError
+from urllib.parse import urlparse
 
 from rucio import version
 from rucio.common import exception
@@ -408,9 +377,8 @@ class BaseClient(object):
 
         if headers is not None:
             hds.update(headers)
-
         if verify is None:
-            verify = self.ca_cert
+            verify = self.ca_cert or False  # Maybe unnecessary but make sure to convert "" -> False
 
         self.logger.debug("HTTP request: %s %s" % (type_, url))
         for h, v in hds.items():
@@ -616,12 +584,7 @@ class BaseClient(object):
                 print("Copy paste the code from the browser to the terminal and press enter:")
                 count = 0
                 while count < 3:
-                    # Python3 default
-                    get_input = input
-                    # if Python version <= 2.7 use raw_input
-                    if sys.version_info[:2] <= (2, 7):
-                        get_input = raw_input  # noqa: F821 pylint: disable=undefined-variable
-                    fetchcode = get_input()
+                    fetchcode = input()
                     fetch_url = build_url(self.auth_host, path='auth/oidc_redirect', params=fetchcode)
                     result = self._send_request(fetch_url, headers=headers, get_token=True)
                     if 'X-Rucio-Auth-Token' in result.headers and result.status_code == codes.ok:
@@ -920,7 +883,6 @@ class BaseClient(object):
             except Exception:
                 raise
 
-        # if the file exists check if the stored token is valid. If not request a new one and overwrite the file. Otherwise use the one from the file
         try:
             file_d, file_n = mkstemp(dir=self.token_path)
             with fdopen(file_d, "w") as f_token:
