@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,22 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2017
-# - Martin Barisits <martin.barisits@cern.ch>, 2012-2021
-# - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2018
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2012-2013
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2020
-# - Wen Guan <wen.guan@cern.ch>, 2014-2015
-# - Tobias Wegner <twegner@cern.ch>, 2018-2019
-# - Frank Berghaus <berghaus@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Eric Vaandering <ewv@fnal.gov>, 2020
 
 """
     Exceptions used with Rucio.
@@ -957,13 +941,13 @@ class DuplicateFileTransferSubmission(RucioException):
         self.error_code = 90
 
 
-class DIDTypeError(RucioException):
+class DIDError(RucioException):
     """
-    An operation related to DIDType went wrong
+    An operation related to DID type went wrong
     """
     def __init__(self, *args, **kwargs):
-        super(DIDTypeError, self).__init__(*args, **kwargs)
-        self._message = 'Error using DIDType'
+        super(DIDError, self).__init__(*args, **kwargs)
+        self._message = 'Error using DID type'
         self.error_code = 91
 
 
@@ -1055,3 +1039,23 @@ class InvalidAlgorithmName(RucioException):
         super(InvalidAlgorithmName, self).__init__(*args, **kwargs)
         self.message = 'Algorithm name %s is not valid for VO %s' % (algorithm, vo)
         self.error_code = 100
+
+
+class FilterEngineGenericError(RucioException):
+    """
+    Generic Filter Engine error.
+    """
+    def __init__(self, *args, **kwargs):
+        super(FilterEngineGenericError, self).__init__(*args, **kwargs)
+        self._message = 'Generic filter engine error.'
+        self.error_code = 101
+
+
+class MetadataSchemaMismatchError(RucioException):
+    """
+    External table does not match expected table schema.
+    """
+    def __init__(self, *args, **kwargs):
+        super(MetadataSchemaMismatchError, self).__init__(*args, **kwargs)
+        self._message = 'The external table does not match the expected table schema.'
+        self.error_code = 102
