@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,41 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2021
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2021
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2022
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2013
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2022
-# - Brian Bockelman <bbockelm@cse.unl.edu>, 2018
-# - Tobias Wegner <twegner@cern.ch>, 2018-2019
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Tomas Javurek <tomas.javurek@cern.ch>, 2019-2020
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2019-2021
-# - Gabriele Fronze' <gfronze@cern.ch>, 2019
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - root <root@escape-rucio-dev-oidc-r.cern.ch>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Mayank Sharma <mayank.sharma@cern.ch>, 2021
-# - Rahul Chauhan <omrahulchauhan@gmail.com>, 2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
-# - Anil Panta <47672624+panta-123@users.noreply.github.com>, 2021
-# - Ilija Vukotic <ivukotic@cern.ch>, 2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - martynia <janusz.martyniak@googlemail.com>, 2021-2022
-# - jdierkes <joel.dierkes@cern.ch>, 2021
-# - Rakshita Varadarajan <rakshitajps@gmail.com>, 2021
-# - Rob Barnsley <robbarnsley@users.noreply.github.com>, 2021
-# - Igor Mandrichenko <ivm@fnal.gov>, 2021
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
-
-from __future__ import absolute_import, print_function
 
 import argparse
 import base64
@@ -75,10 +40,10 @@ from uuid import uuid4 as uuid
 from xml.etree import ElementTree
 
 import requests
-from six import string_types, text_type, binary_type, ensure_text, PY3
-from six.moves import StringIO, zip_longest as izip_longest
-from six.moves.urllib.parse import urlparse, urlencode, quote, parse_qsl, urlunparse
-from six.moves.configparser import NoOptionError, NoSectionError
+from io import StringIO
+from itertools import zip_longest
+from urllib.parse import urlparse, urlencode, quote, parse_qsl, urlunparse
+from configparser import NoOptionError, NoSectionError
 
 from rucio.common.config import config_get, config_has_section
 from rucio.common.exception import MissingModuleException, InvalidType, InputValidationError, MetalinkJsonParsingError, RucioException, \
@@ -195,7 +160,7 @@ def all_oidc_req_claims_present(scope, audience, required_scope, required_audien
         req_scope_present = all(elem in scope for elem in required_scope)
         req_audience_present = all(elem in audience for elem in required_audience)
         return req_scope_present and req_audience_present
-    elif (isinstance(scope, string_types) and isinstance(audience, string_types) and isinstance(required_scope, string_types) and isinstance(required_audience, string_types)):
+    elif (isinstance(scope, str) and isinstance(audience, str) and isinstance(required_scope, str) and isinstance(required_audience, str)):
         scope = str(scope)
         audience = str(audience)
         required_scope = str(required_scope)
@@ -203,7 +168,7 @@ def all_oidc_req_claims_present(scope, audience, required_scope, required_audien
         req_scope_present = all(elem in scope.split(sepatator) for elem in required_scope.split(sepatator))
         req_audience_present = all(elem in audience.split(sepatator) for elem in required_audience.split(sepatator))
         return req_scope_present and req_audience_present
-    elif (isinstance(scope, list) and isinstance(audience, list) and isinstance(required_scope, string_types) and isinstance(required_audience, string_types)):
+    elif (isinstance(scope, list) and isinstance(audience, list) and isinstance(required_scope, str) and isinstance(required_audience, str)):
         scope = [str(it) for it in scope]
         audience = [str(it) for it in audience]
         required_scope = str(required_scope)
@@ -211,7 +176,7 @@ def all_oidc_req_claims_present(scope, audience, required_scope, required_audien
         req_scope_present = all(elem in scope for elem in required_scope.split(sepatator))
         req_audience_present = all(elem in audience for elem in required_audience.split(sepatator))
         return req_scope_present and req_audience_present
-    elif (isinstance(scope, string_types) and isinstance(audience, string_types) and isinstance(required_scope, list) and isinstance(required_audience, list)):
+    elif (isinstance(scope, str) and isinstance(audience, str) and isinstance(required_scope, list) and isinstance(required_audience, list)):
         scope = str(scope)
         audience = str(audience)
         required_scope = [str(it) for it in required_scope]
@@ -263,14 +228,6 @@ def set_preferred_checksum(checksum_name):
         PREFERRED_CHECKSUM = checksum_name
 
 
-def set_checksum_value(file, checksum_names_list):
-    for checksum_name in checksum_names_list:
-        if checksum_name in file['metadata'].keys() and file['metadata'][checksum_name]:
-            file['checksum'] = '%s:%s' % (checksum_name.upper(), str(file['metadata'][checksum_name]))
-            if checksum_name == PREFERRED_CHECKSUM:
-                break
-
-
 def adler32(file):
     """
     An Adler-32 checksum is obtained by calculating two 16-bit checksums A and B
@@ -285,11 +242,11 @@ def adler32(file):
     adler = 1
 
     can_mmap = False
-    try:
-        with open(file, 'r+b') as f:
-            can_mmap = True
-    except:
-        pass
+    # try:
+    #    with open(file, 'r+b') as f:
+    #        can_mmap = True
+    # except:
+    #    pass
 
     try:
         # use mmap if possible
@@ -387,11 +344,11 @@ def val_to_space_sep_str(vallist):
     """
     try:
         if isinstance(vallist, list):
-            return text_type(" ".join(vallist))
+            return str(" ".join(vallist))
         else:
-            return text_type(vallist)
+            return str(vallist)
     except:
-        return text_type('')
+        return str('')
 
 
 def date_to_str(date):
@@ -440,7 +397,7 @@ def datetime_parser(dct):
     """ datetime parser
     """
     for k, v in list(dct.items()):
-        if isinstance(v, string_types) and re.search(" UTC", v):
+        if isinstance(v, str) and re.search(" UTC", v):
             try:
                 dct[k] = datetime.datetime.strptime(v, DATE_FORMAT)
             except Exception:
@@ -484,7 +441,7 @@ def execute(cmd, blocking=True):
 
 def rse_supported_protocol_operations():
     """ Returns a list with operations supported by all RSE protocols."""
-    return ['read', 'write', 'delete', 'third_party_copy', 'third_party_copy_read', 'third_party_copy_write']
+    return ['read', 'write', 'delete', 'third_party_copy_read', 'third_party_copy_write']
 
 
 def rse_supported_protocol_domains():
@@ -496,7 +453,7 @@ def grouper(iterable, n, fillvalue=None):
     """ Collect data into fixed-length chunks or blocks """
     # grouper('ABCDEFG', 3, 'x') --> ABC DEF Gxx
     args = [iter(iterable)] * n
-    return izip_longest(*args, fillvalue=fillvalue)
+    return zip_longest(*args, fillvalue=fillvalue)
 
 
 def chunks(iterable, n):
@@ -536,7 +493,7 @@ def my_key_generator(namespace, fn, **kw):
     return generate_key
 
 
-def construct_surl_DQ2(dsn, filename):
+def construct_surl_DQ2(dsn: str, scope: str, filename: str) -> str:
     """
     Defines relative SURL for new replicas. This method
     contains DQ2 convention. To be used for non-deterministic sites.
@@ -575,7 +532,7 @@ def construct_surl_DQ2(dsn, filename):
         return '/%s/%s/%s/%s/%s' % (project, dataset_type, tag, stripped_dsn, filename)
 
 
-def construct_surl_T0(dsn, filename):
+def construct_surl_T0(dsn: str, scope: str, filename: str) -> str:
     """
     Defines relative SURL for new replicas. This method
     contains Tier0 convention. To be used for non-deterministic sites.
@@ -595,7 +552,7 @@ def construct_surl_T0(dsn, filename):
         return '/other/other/other/other/%s' % (filename)
 
 
-def construct_surl_BelleII(dsn, filename):
+def construct_surl_BelleII(dsn: str, scope: str, filename: str) -> str:
     """
     Defines relative SURL for Belle II specific replicas.
     This method contains the Belle II convention.
@@ -628,7 +585,14 @@ register_surl_algorithm(construct_surl_DQ2, 'DQ2')
 register_surl_algorithm(construct_surl_BelleII, 'BelleII')
 
 
-def construct_surl(dsn, filename, naming_convention=None):
+def construct_surl(dsn: str, scope: str, filename: str, naming_convention: str = None) -> str:
+    """
+    Applies non-deterministic source url convention to the given replica.
+    use the naming_convention to call the actual function which will do the job.
+    Rucio administrators can potentially register additional surl generation algorithms,
+    which are not implemented inside this main rucio repository, so changing the
+    argument list must be done with caution.
+    """
     global _loaded_policy_modules
     if not _loaded_policy_modules:
         # on first call, register any SURL functions from the policy packages
@@ -637,7 +601,7 @@ def construct_surl(dsn, filename, naming_convention=None):
 
     if naming_convention is None or naming_convention not in _SURL_ALGORITHMS:
         naming_convention = _DEFAULT_SURL
-    return _SURL_ALGORITHMS[naming_convention](dsn, filename)
+    return _SURL_ALGORITHMS[naming_convention](dsn, scope, filename)
 
 
 def __strip_dsn(dsn):
@@ -729,6 +693,8 @@ def extract_scope_dirac(did, scopes):
 
 def extract_scope_belleii(did, scopes):
     split_did = did.split('/')
+    if did.startswith('/belle/mock/'):
+        return 'mock', did
     if did.startswith('/belle/MC/'):
         if did.startswith('/belle/MC/BG') or \
            did.startswith('/belle/MC/build') or \
@@ -816,7 +782,7 @@ def extract_scope(did, scopes=None, default_extract=_DEFAULT_EXTRACT):
     if not _loaded_policy_package_scope_algorithms:
         register_policy_package_algorithms('scope', _EXTRACT_SCOPE_ALGORITHMS)
         _loaded_policy_package_scope_algorithms = True
-    extract_scope_convention = config_get('common', 'extract_scope', False, None)
+    extract_scope_convention = config_get('common', 'extract_scope', False, None) or config_get('policy', 'extract_scope', False, None)
     if extract_scope_convention is None or extract_scope_convention not in _EXTRACT_SCOPE_ALGORITHMS:
         extract_scope_convention = default_extract
     return _EXTRACT_SCOPE_ALGORITHMS[extract_scope_convention](did=did, scopes=scopes)
@@ -989,7 +955,7 @@ def ssh_sign(private_key, message):
     :param message: The message to sign as a string.
     :return: Base64 encoded signature as a string.
     """
-    if PY3 and isinstance(message, str):
+    if isinstance(message, str):
         message = message.encode()
     if not EXTRA_MODULES['paramiko']:
         raise MissingModuleException('The paramiko module is not installed or faulty.')
@@ -999,8 +965,7 @@ def ssh_sign(private_key, message):
     signature_stream = priv_k.sign_ssh_data(message)
     signature_stream.rewind()
     base64_encoded = base64.b64encode(signature_stream.get_remainder())
-    if PY3:
-        base64_encoded = base64_encoded.decode()
+    base64_encoded = base64_encoded.decode()
     return base64_encoded
 
 
@@ -1139,7 +1104,7 @@ def parse_did_filter_from_string(input_string):
                 except ValueError:
                     raise ValueError('Length has to be an integer value.')
                 filters[key] = value
-            elif isinstance(value, string_types):
+            elif isinstance(value, str):
                 if value.lower() == 'true':
                     value = '1'
                 elif value.lower() == 'false':
@@ -1385,7 +1350,7 @@ def run_cmd_process(cmd, timeout=3600):
     """
 
     time_start = datetime.datetime.now().second
-    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, preexec_fn=os.setsid)
+    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, preexec_fn=os.setsid, universal_newlines=True)
 
     running_time = 0
     while process.poll() is None and running_time < timeout:
@@ -1399,9 +1364,6 @@ def run_cmd_process(cmd, timeout=3600):
         process.kill()
 
     stdout, stderr = process.communicate()
-    if isinstance(stdout, binary_type):
-        stdout = ensure_text(stdout, errors='replace')
-        stderr = ensure_text(stderr, errors='replace')
     if not stderr:
         stderr = ''
     if not stdout:
@@ -1597,7 +1559,7 @@ class retry:
     def __init__(self, func, *args, **kwargs):
         '''
         :param func: a method that should be executed with retries
-        :param args parametres of the func
+        :param args: parametres of the func
         :param kwargs: key word arguments of the func
         '''
         self.func, self.args, self.kwargs = func, args, kwargs
@@ -1862,3 +1824,75 @@ def register_policy_package_algorithms(algorithm_type, dictionary):
             vos = list_vos()
             for vo in vos:
                 try_importing_policy(algorithm_type, dictionary, vo['vo'])
+
+
+class Availability:
+    """
+    This util class acts as a translator between the availability stored as
+    integer and as boolen values.
+
+    `None` represents a missing value. This lets a user update a specific value
+    without altering the other ones. If it needs to be evaluated, it will
+    correspond to `True`.
+    """
+
+    read = None
+    write = None
+    delete = None
+
+    def __init__(self, read=None, write=None, delete=None):
+        self.read = read
+        self.write = write
+        self.delete = delete
+
+    def __iter__(self):
+        """
+        The iterator provides the feature to unpack the values of this class.
+
+        e.g. `read, write, delete = Availability(True, False, True)`
+
+        :returns: An iterator over the values `read`, `write`, `delete`.
+        """
+        return iter((self.read, self.write, self.delete))
+
+    def __repr__(self):
+        return "Availability({}, {}, {})".format(self.read, self.write, self.delete)
+
+    def __eq__(self, other):
+        return self.read == other.read and self.write == other.write and self.delete == other.delete
+
+    def __hash__(self):
+        return hash(self.integer)
+
+    @classmethod
+    def from_integer(cls, n):
+        """
+        Returns a new Availability instance where the values are set to the
+        corresponding bit values in the integer.
+
+        :param n: The integer value to get the availabilities from.
+        :returns: The corresponding Availability instance.
+        """
+        if n is None:
+            return cls(None, None, None)
+
+        return cls(
+            (n >> 2) % 2 == 1,
+            (n >> 1) % 2 == 1,
+            (n >> 0) % 2 == 1
+        )
+
+    @property
+    def integer(self):
+        """
+        Returns the corresponding integer for the instance values. The three
+        least-significant bits correspond to the availability values.
+
+        :returns: An integer corresponding to the availability values. `None`
+            gets treated as `True`.
+        """
+        read_value = (self.read or self.read is None) * 4
+        write_value = (self.write or self.write is None) * 2
+        delete_value = (self.delete or self.delete is None) * 1
+
+        return read_value + write_value + delete_value
