@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,21 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2012
-# - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2020
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2015
-# - Martin Barisits <martin.barisits@cern.ch>, 2014-2019
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2019
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 from datetime import datetime
 from enum import Enum
@@ -47,8 +32,6 @@ from rucio.db.sqla import models
 from rucio.db.sqla.constants import AccountStatus, AccountType
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
 
-from six import string_types
-
 
 @transactional_session
 def add_account(account, type_, email, session=None):
@@ -142,7 +125,7 @@ def update_account(account, key, value, session=None):
     except exc.NoResultFound:
         raise exception.AccountNotFound('Account with ID \'%s\' cannot be found' % account)
     if key == 'status':
-        if isinstance(value, string_types):
+        if isinstance(value, str):
             value = AccountStatus[value]
         if value == AccountStatus.SUSPENDED:
             query.update({'status': value, 'suspended_at': datetime.utcnow()})
@@ -153,7 +136,7 @@ def update_account(account, key, value, session=None):
 
 
 @stream_session
-def list_accounts(filter_={}, session=None):
+def list_accounts(filter_=None, session=None):
     """ Returns a list of all account names.
 
     :param filter_: Dictionary of attributes by which the input data should be filtered
@@ -161,11 +144,13 @@ def list_accounts(filter_={}, session=None):
 
     returns: a list of all account names.
     """
+    if filter_ is None:
+        filter_ = {}
     query = session.query(models.Account.account, models.Account.account_type,
                           models.Account.email).filter_by(status=AccountStatus.ACTIVE)
     for filter_type in filter_:
         if filter_type == 'account_type':
-            if isinstance(filter_['account_type'], string_types):
+            if isinstance(filter_['account_type'], str):
                 query = query.filter_by(account_type=AccountType[filter_['account_type']])
             elif isinstance(filter_['account_type'], Enum):
                 query = query.filter_by(account_type=filter_['account_type'])
