@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,22 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2021
-# - Martin Barisits <martin.barisits@cern.ch>, 2012-2019
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2017
-# - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2013-2014
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2017
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Rahul Chauhan <omrahulchauhan@gmail.com>, 2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 import datetime
 import hashlib
@@ -37,10 +21,12 @@ import traceback
 from base64 import b64decode
 
 import paramiko
-from dogpile.cache import make_region
 from dogpile.cache.api import NO_VALUE
+from dogpile.cache import make_region
 from sqlalchemy import and_, or_, select, delete
 
+from rucio.common.cache import make_region_memcached
+from rucio.common.config import config_get_bool
 from rucio.common.exception import CannotAuthenticate, RucioException
 from rucio.common.utils import chunks, generate_uuid
 from rucio.core.account import account_exists
@@ -59,12 +45,10 @@ def token_key_generator(namespace, fni, **kwargs):
     return generate_key
 
 
-TOKENREGION = make_region(
-    function_key_generator=token_key_generator
-).configure(
-    'dogpile.cache.memory',
-    expiration_time=3600
-)
+if config_get_bool('cache', 'use_external_cache_for_auth_tokens', default=False):
+    TOKENREGION = make_region_memcached(expiration_time=900, function_key_generator=token_key_generator)
+else:
+    TOKENREGION = make_region(function_key_generator=token_key_generator).configure('dogpile.cache.memory', expiration_time=900)
 
 
 @transactional_session
