@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,20 +12,13 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 from abc import ABCMeta, abstractmethod
 
-from six import add_metaclass
-
 from rucio.db.sqla.session import transactional_session
 
 
-@add_metaclass(ABCMeta)
-class DidMetaPlugin(object):
+class DidMetaPlugin(object, metaclass=ABCMeta):
     """
     Interface for plugins managing metadata of DIDs
     """
