@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2020 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,16 +12,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2019
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Tomas Javurek <tomas.javurek@cern.ch>, 2020
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2020
 
-from six import string_types
 from rucio.common.exception import RSEOperationNotSupported
 from rucio.common.types import InternalAccount
 from rucio.core import rse as rse_module, distance as distance_module, account as account_module, identity as identity_module
@@ -36,20 +27,22 @@ def import_rses(rses, rse_sync_method='edit', attr_sync_method='edit', protocol_
     new_rses = []
     for rse_name in rses:
         rse = rses[rse_name]
-        if isinstance(rse.get('rse_type'), string_types):
+        if isinstance(rse.get('rse_type'), str):
             rse['rse_type'] = RSEType(rse['rse_type'])
 
         if rse_module.rse_exists(rse_name, vo=vo, include_deleted=False, session=session):
             # RSE exists and is active
             rse_id = rse_module.get_rse_id(rse=rse_name, vo=vo, session=session)
-            rse_module.update_rse(rse_id=rse_id, parameters=rse, session=session)
+            selected_rse_properties = {key: rse[key] for key in rse if key in rse_module.MUTABLE_RSE_PROPERTIES}
+            rse_module.update_rse(rse_id=rse_id, parameters=selected_rse_properties, session=session)
         elif rse_module.rse_exists(rse_name, vo=vo, include_deleted=True, session=session):
             # RSE exists but in deleted state
             # Should only modify the RSE if importer is configured for edit or hard sync
             if rse_sync_method in ['edit', 'hard']:
                 rse_id = rse_module.get_rse_id(rse=rse_name, vo=vo, include_deleted=True, session=session)
                 rse_module.restore_rse(rse_id, session=session)
-                rse_module.update_rse(rse_id=rse_id, parameters=rse, session=session)
+                selected_rse_properties = {key: rse[key] for key in rse if key in rse_module.MUTABLE_RSE_PROPERTIES}
+                rse_module.update_rse(rse_id=rse_id, parameters=selected_rse_properties, session=session)
             else:
                 # Config is in RSE append only mode, should not modify the disabled RSE
                 continue
