@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2020 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,18 +12,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2015
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2020
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-# - Martin Barisits <martin.barisits@cern.ch>, 2020
 
-from __future__ import print_function
 from re import match
-from six import string_types
 
 from sqlalchemy.exc import IntegrityError
 from sqlalchemy.orm.exc import NoResultFound
@@ -54,7 +44,7 @@ def add_key(key, key_type, value_type=None, value_regexp=None, session=None):
         raise UnsupportedValueType('The type \'%(value_type)s\' is not supported for values!' % locals())
 
     # Convert key_type
-    if isinstance(key_type, string_types):
+    if isinstance(key_type, str):
         key_type = str(key_type)
     else:
         key_type = str(key_type.value)
