@@ -19,10 +19,9 @@ import logging
 import traceback
 from io import StringIO
 from re import match
-from typing import Any, Dict, Optional, Sequence
+from typing import Any, Dict, List, Optional, Sequence, Union
 
 import sqlalchemy
-import sqlalchemy.orm
 from dogpile.cache.api import NO_VALUE
 from sqlalchemy.exc import DatabaseError, IntegrityError, OperationalError
 from sqlalchemy.orm import aliased, Session
@@ -40,6 +39,7 @@ from rucio.db.sqla.constants import (RSEType, ReplicaState)
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
 
 
+RSE_SETTINGS = ["continent", "city", "region_code", "country_name", "time_zone", "ISP", "ASN"]
 REGION = make_region_memcached(expiration_time=900)
 
 
@@ -157,9 +157,6 @@ class RseData:
                 rse_data.limits = limits
 
 
-RSE_SETTINGS = ["continent", "city", "region_code", "country_name", "time_zone", "ISP", "ASN"]
-
-
 @transactional_session
 def add_rse(rse, vo='def', deterministic=True, volatile=False, city=None, region_code=None, country_name=None, continent=None, time_zone=None,
             ISP=None, staging_area=False, rse_type=RSEType.DISK, longitude=None, latitude=None, ASN=None, availability: Optional[int] = 7, session=None):
@@ -693,39 +690,51 @@ def get_rses_with_attribute_value(key, value, lookup_key, vo='def', session=None
 
 
 @read_session
-def get_rse_attribute(key, rse_id=None, value=None, use_cache=True, session=None):
+def _get_rse_attribute_without_cache(rse_id: str, key: str, session: Optional[Session] = None) -> Optional[Union[str, bool]]:
     """
-    Retrieve RSE attribute value.
+    Retrieve RSE attribute value from the database.
 
     :param rse_id: The RSE id.
     :param key: The key for the attribute.
-    :param value: Optionally, the desired value for the attribute.
-    :param use_cache: Boolean to use memcached.
     :param session: The database session in use.
 
-    :returns: A list with RSE attribute values for a Key.
+    :returns: The value for the rse attribute, None if it does not exist.
     """
 
-    result = NO_VALUE
+    value = session.query(models.RSEAttrAssociation.value).filter_by(rse_id=rse_id, key=key).one_or_none()
+
+    if value is None:
+        return None
+
+    return value.value
+
+
+@read_session
+def get_rse_attribute(rse_id: str, key: str, use_cache: bool = True, session: Optional[Session] = None) -> Optional[Union[str, bool]]:
+    """
+    Retrieve RSE attribute value. If it is not cached, look it up in the
+    database. If the value exists and is not cached, it will be added to the
+    cache.
+
+    :param rse_id: The RSE id.
+    :param key: The key for the attribute.
+    :param session: The database session in use.
+
+    :returns: The value for the rse attribute, None if it does not exist.
+    """
+    cache_key = f'rse_attributes_{rse_id}_{key}'
     if use_cache:
-        result = REGION.get('%s-%s-%s' % (key, rse_id, value))
-    if result is NO_VALUE:
+        value = REGION.get(cache_key)
 
-        rse_attrs = []
-        if rse_id:
-            query = session.query(models.RSEAttrAssociation.value).filter_by(rse_id=rse_id, key=key).distinct()
-            if value:
-                query = session.query(models.RSEAttrAssociation.value).filter_by(rse_id=rse_id, key=key, value=value).distinct()
-        else:
-            query = session.query(models.RSEAttrAssociation.value).filter_by(key=key).distinct()
-            if value:
-                query = session.query(models.RSEAttrAssociation.value).filter_by(key=key, value=value).distinct()
-        for attr_value in query:
-            rse_attrs.append(attr_value[0])
-        REGION.set('%s-%s-%s' % (key, rse_id, value), rse_attrs)
-        return rse_attrs
+        if value is not NO_VALUE:
+            return value
 
-    return result
+    value = _get_rse_attribute_without_cache(rse_id, key, session=session)
+
+    if use_cache:
+        REGION.set(cache_key, value)
+
+    return value
 
 
 def get_rse_attributes(rse_id, session=None):
@@ -751,15 +760,16 @@ def get_rse_attributes(rse_id, session=None):
     return result
 
 
-def get_rse_supported_checksums_from_attributes(rse_attributes):
+def get_rse_supported_checksums_from_attributes(rse_attributes: Dict[str, Any]) -> List[str]:
     """
     Parse the RSE attribute defining the checksum supported by the RSE
     :param rse_attributes: attributes retrieved using list_rse_attributes
+    :returns: A list of the names of supported checksums indicated by the specified attributes.
     """
-    return parse_checksum_support_attribute(rse_attributes.get(CHECKSUM_KEY))
+    return parse_checksum_support_attribute(rse_attributes.get(CHECKSUM_KEY, ''))
 
 
-def parse_checksum_support_attribute(checksum_attribute):
+def parse_checksum_support_attribute(checksum_attribute: str) -> List[str]:
     """
     Parse the checksum support RSE attribute.
     :param checksum_attribute: The value of the RSE attribute storing the checksum value
@@ -771,10 +781,12 @@ def parse_checksum_support_attribute(checksum_attribute):
 
     if not checksum_attribute:
         return GLOBALLY_SUPPORTED_CHECKSUMS
+
+    supported_checksum_list = [c.strip() for c in checksum_attribute.split(',') if c.strip()]
+
+    if 'none' in supported_checksum_list:
+        return []
     else:
-        supported_checksum_list = [c.strip() for c in checksum_attribute.split(',') if c.strip()]
-        if 'none' in supported_checksum_list:
-            return []
         return supported_checksum_list
 
 
@@ -1096,18 +1108,17 @@ def get_rse_protocols(rse_id, schemes=None, session=None):
     if not _rse:
         raise exception.RSENotFound('RSE with id \'%s\' not found' % rse_id)
 
-    lfn2pfn_algorithms = get_rse_attribute('lfn2pfn_algorithm', rse_id=_rse.id, session=session)
+    lfn2pfn_algorithm = get_rse_attribute(_rse.id, 'lfn2pfn_algorithm', session=session)
     # Resolve LFN2PFN default algorithm as soon as possible.  This way, we can send back the actual
     # algorithm name in response to REST queries.
-    lfn2pfn_algorithm = get_lfn2pfn_algorithm_default()
-    if lfn2pfn_algorithms:
-        lfn2pfn_algorithm = lfn2pfn_algorithms[0]
+    if not lfn2pfn_algorithm:
+        lfn2pfn_algorithm = get_lfn2pfn_algorithm_default()
 
     # Copy verify_checksum from the attributes, later: assume True if not specified
-    verify_checksum = get_rse_attribute('verify_checksum', rse_id=_rse.id, session=session)
+    verify_checksum = get_rse_attribute(_rse.id, 'verify_checksum', session=session)
 
     # Copy sign_url from the attributes
-    sign_url = get_rse_attribute('sign_url', rse_id=_rse.id, session=session)
+    sign_url = get_rse_attribute(_rse.id, 'sign_url', session=session)
 
     read = True if _rse.availability & 4 else False
     write = True if _rse.availability & 2 else False
@@ -1125,9 +1136,9 @@ def get_rse_protocols(rse_id, schemes=None, session=None):
             'qos_class': _rse.qos_class,
             'rse': _rse.rse,
             'rse_type': _rse.rse_type.name,
-            'sign_url': sign_url[0] if sign_url else None,
+            'sign_url': sign_url,
             'staging_area': _rse.staging_area,
-            'verify_checksum': verify_checksum[0] if verify_checksum else True,
+            'verify_checksum': verify_checksum if verify_checksum is not None else True,
             'volatile': _rse.volatile}
 
     for op in utils.rse_supported_protocol_operations():
