@@ -26,7 +26,7 @@ from rucio.core import account_counter, rse_counter, request as request_core
 import rucio.core.did
 import rucio.core.lock
 import rucio.core.replica
-from rucio.core.rse import get_rse
+from rucio.core.rse import get_rse, get_rse_attribute, get_rse_name
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import LockState, RuleGrouping, ReplicaState, RequestType, DIDType, OBSOLETE
 from rucio.db.sqla.session import transactional_session
@@ -825,10 +825,14 @@ def __create_lock_and_replica(file, dataset, rule, rse_id, staging_area, availab
     :attention:                  This method modifies the contents of the locks, locks_to_create, replicas_to_create and replicas input parameters.
     """
 
-    # If it is a Staging Area, the pin has to be extended
-    if staging_area:
+    if rule.expires_at:
         lifetime = rule.expires_at - datetime.utcnow()
         lifetime = lifetime.seconds + lifetime.days * 24 * 3600
+    else:
+        lifetime = None
+
+    # If it is a Staging Area, the pin has to be extended
+    if staging_area:
         transfers_to_create.append(create_transfer_dict(dest_rse_id=rse_id,
                                                         request_type=RequestType.STAGEIN,
                                                         scope=file['scope'],
@@ -842,6 +846,16 @@ def __create_lock_and_replica(file, dataset, rule, rse_id, staging_area, availab
                                                         lifetime=lifetime,
                                                         session=session))
 
+    # If staging_required type RSE then set pin to RSE attribute maximum_pin_lifetime
+    staging_required = get_rse_attribute(rse_id, 'staging_required', session=session)
+    maximum_pin_lifetime = get_rse_attribute(rse_id, 'maximum_pin_lifetime', session=session)
+
+    if staging_required:
+        if (not lifetime and maximum_pin_lifetime) or (lifetime and maximum_pin_lifetime and lifetime < int(maximum_pin_lifetime)):
+            lifetime = maximum_pin_lifetime
+        rse_name = get_rse_name(rse_id=rse_id, session=session)
+        logger(logging.DEBUG, f'Destination RSE {rse_name} is type staging_required with pin value: {lifetime}')
+
     existing_replicas = [replica for replica in replicas[(file['scope'], file['name'])] if replica.rse_id == rse_id]
 
     if existing_replicas:  # A replica already exists (But could be UNAVAILABLE)
@@ -855,12 +869,27 @@ def __create_lock_and_replica(file, dataset, rule, rse_id, staging_area, availab
                                      name=file['name'],
                                      bytes_=file['bytes'],
                                      existing_replica=existing_replica,
-                                     state=LockState.OK)
+                                     state=LockState.OK if not staging_required else LockState.REPLICATING)
             if rse_id not in locks_to_create:
                 locks_to_create[rse_id] = []
             locks_to_create[rse_id].append(new_lock)
             locks[(file['scope'], file['name'])].append(new_lock)
-            return False
+            if not staging_required:
+                return False
+
+            transfers_to_create.append(create_transfer_dict(dest_rse_id=rse_id,
+                                                            request_type=RequestType.STAGEIN,
+                                                            scope=file['scope'],
+                                                            name=file['name'],
+                                                            rule=rule,
+                                                            lock=new_lock,
+                                                            bytes_=file['bytes'],
+                                                            md5=file['md5'],
+                                                            adler32=file['adler32'],
+                                                            ds_scope=dataset['scope'],
+                                                            ds_name=dataset['name'],
+                                                            lifetime=lifetime,
+                                                            session=session))
 
         # Replica is not available -- UNAVAILABLE
         elif existing_replica.state == ReplicaState.UNAVAILABLE:
@@ -893,6 +922,7 @@ def __create_lock_and_replica(file, dataset, rule, rse_id, staging_area, availab
                                                                 adler32=file['adler32'],
                                                                 ds_scope=dataset['scope'],
                                                                 ds_name=dataset['name'],
+                                                                lifetime=lifetime,
                                                                 session=session))
                 return True
             return False
@@ -954,6 +984,7 @@ def __create_lock_and_replica(file, dataset, rule, rse_id, staging_area, availab
                                                             adler32=file['adler32'],
                                                             ds_scope=dataset['scope'],
                                                             ds_name=dataset['name'],
+                                                            lifetime=lifetime,
                                                             session=session))
             return True
         return False
