@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2020 CERN for the benefit of the ATLAS collaboration.
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,28 +12,13 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Fernando López <felopez@cern.ch>, 2015
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2017-2018
-# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-#
-# PY3K COMPATIBLE
 
 from rucio.common.config import get_config_dirs
 from rucio.common.dumper import DUMPS_CACHE_DIR
 from rucio.common.dumper import http_download_to_file, gfal_download_to_file, ddmendpoint_url, temp_file
 
-try:
-    import ConfigParser
-except ImportError:
-    import configparser as ConfigParser
-try:
-    import HTMLParser
-except ImportError:
-    import html.parser as HTMLParser
+import configparser as ConfigParser
+import html.parser as HTMLParser
 import datetime
 import glob
 import hashlib
