@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,22 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Brandon White <bjwhite@fnal.gov>, 2019
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2020
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2022
-
-from __future__ import division
 
 import logging
 import random
@@ -230,7 +214,7 @@ def automatix(sites, inputfile, sleep_time, account, worker_number=1, total_work
         for site in sites:
 
             for retry in range(0, totretries):
-                start_time = time()
+                timer = monitor.Timer()
                 tmpdir = tempfile.mkdtemp()
                 logger(logging.INFO, 'Running on site %s', site)
                 dic = choose_element(probabilities, data)
@@ -266,7 +250,7 @@ def automatix(sites, inputfile, sleep_time, account, worker_number=1, total_work
                 if status:
                     monitor.record_counter(name='automatix.addnewdataset.done', delta=1)
                     monitor.record_counter(name='automatix.addnewfile.done', delta=nbfiles)
-                    monitor.record_timer('automatix.datasetinjection', (time() - start_time) * 1000)
+                    timer.record('automatix.datasetinjection')
                     break
                 else:
                     logger(logging.INFO, 'Failed to upload files. Will retry another time (attempt %s/%s)', str(retry + 1), str(totretries))
