@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,38 +12,24 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Wen Guan <wen.guan@cern.ch>, 2015-2016
-# - Martin Barisits <martin.barisits@cern.ch>, 2015-2017
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2016-2018
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2017-2021
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2018-2019
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
-# - Brandon White <bjwhite@fnal.gov>, 2019
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
 
 """
 Conveyor stager is a daemon to manage stagein file transfers.
 """
 
-from __future__ import division
-
 import functools
 import logging
 import threading
-import time
 
-from six.moves.configparser import NoOptionError
+from configparser import NoOptionError
 
 import rucio.db.sqla.util
 from rucio.common import exception
 from rucio.common.config import config_get, config_get_bool, config_get_int
 from rucio.common.logging import setup_logging
-from rucio.core.monitor import record_counter, record_timer
-from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses, run_conveyor_daemon, next_transfers_to_submit
+from rucio.core.monitor import record_counter, record_timer, Timer
+from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses, next_transfers_to_submit
+from rucio.daemons.common import run_daemon
 from rucio.db.sqla.constants import RequestType
 from rucio.transfertool.fts3 import FTS3Transfertool
 
@@ -53,7 +39,7 @@ graceful_stop = threading.Event()
 def run_once(bulk, group_bulk, rse_ids, scheme, failover_scheme, transfertool_kwargs, heartbeat_handler, activity):
     worker_number, total_workers, logger = heartbeat_handler.live()
 
-    start_time = time.time()
+    timer = Timer()
     transfers = next_transfers_to_submit(
         total_workers=total_workers,
         worker_number=worker_number,
@@ -62,13 +48,13 @@ def run_once(bulk, group_bulk, rse_ids, scheme, failover_scheme, transfertool_kw
         activity=activity,
         rses=rse_ids,
         schemes=scheme,
-        transfertools_by_name={'fts3': FTS3Transfertool},
+        transfertool_classes=[FTS3Transfertool],
         older_than=None,
         request_type=RequestType.STAGEIN,
         logger=logger,
     )
     total_transfers = len(list(hop for paths in transfers.values() for path in paths for hop in path))
-    record_timer('daemons.conveyor.stager.get_stagein_transfers.per_transfer', (time.time() - start_time) * 1000 / (total_transfers if transfers else 1))
+    timer.record('daemons.conveyor.stager.get_stagein_transfers.per_transfer', divisor=total_transfers or 1)
     record_counter('daemons.conveyor.stager.get_stagein_transfers', total_transfers)
     record_timer('daemons.conveyor.stager.get_stagein_transfers.transfers', total_transfers)
     logger(logging.INFO, 'Got %s stagein transfers for %s' % (total_transfers, activity))
@@ -76,9 +62,9 @@ def run_once(bulk, group_bulk, rse_ids, scheme, failover_scheme, transfertool_kw
     for builder, transfer_paths in transfers.items():
         transfertool_obj = builder.make_transfertool(logger=logger, **transfertool_kwargs.get(builder.transfertool_class, {}))
         logger(logging.INFO, 'Starting to group transfers for %s (%s)' % (activity, transfertool_obj))
-        start_time = time.time()
-        grouped_jobs = transfertool_obj.group_into_submit_jobs(transfer_paths)
-        record_timer('daemons.conveyor.stager.bulk_group_transfer', (time.time() - start_time) * 1000 / (len(transfer_paths) or 1))
+
+        with Timer('daemons.conveyor.stager.bulk_group_transfer', divisor=len(transfer_paths) or 1):
+            grouped_jobs = transfertool_obj.group_into_submit_jobs(transfer_paths)
 
         logger(logging.INFO, 'Starting to submit transfers for %s (%s)' % (activity, transfertool_obj))
         for job in grouped_jobs:
@@ -147,7 +133,7 @@ def stager(once=False, rses=None, bulk=100, group_bulk=1, group_policy='rule',
         }
     }
 
-    run_conveyor_daemon(
+    run_daemon(
         once=once,
         graceful_stop=graceful_stop,
         executable=executable,
