@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2020 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,11 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2015-2017
-# - Martin Barisits <martin.barisits@cern.ch>, 2016
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2019-2020
 
 ''' add didtype_chck to requests '''
 
@@ -41,7 +36,10 @@ def upgrade():
 
     if context.get_context().dialect.name in ['oracle', 'mysql']:  # pylint: disable=no-member
         add_column('requests', sa.Column('did_type',
-                                         sa.Enum(DIDType, name='REQUESTS_DIDTYPE_CHK', values_callable=lambda obj: [e.value for e in obj]),
+                                         sa.Enum(DIDType,
+                                                 name='REQUESTS_DIDTYPE_CHK',
+                                                 create_constraint=True,
+                                                 values_callable=lambda obj: [e.value for e in obj]),
                                          default=DIDType.FILE), schema=schema[:-1])
         # we don't want checks on the history table, fake the DID type
         add_column('requests_history', sa.Column('did_type', sa.String(1)), schema=schema[:-1])
