@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,19 +12,9 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2015
-# - asket <asket.agarwal96@gmail.com>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2018-2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 
 import uuid
 
-from six import string_types
-
 from sqlalchemy.dialects.postgresql import UUID, JSONB
 from sqlalchemy.dialects.oracle import RAW, CLOB
 from sqlalchemy.dialects.mysql import BINARY
@@ -108,7 +98,7 @@ class BooleanString(TypeDecorator):
             if value:
                 return 'true'
             return 'false'
-        elif isinstance(value, string_types):
+        elif isinstance(value, str):
             if value.lower() == 'true':
                 return 'true'
             elif value.lower() == 'false':
@@ -166,7 +156,7 @@ class InternalAccountString(TypeDecorator):
         if value is None:
             return value
 
-        if isinstance(value, string_types):
+        if isinstance(value, str):
             raise InvalidType('Cannot insert to db. Expected InternalAccount, got string type.')
         else:
             return value.internal
@@ -199,7 +189,7 @@ class InternalScopeString(TypeDecorator):
         if value is None:
             return value
 
-        if isinstance(value, string_types):
+        if isinstance(value, str):
             raise InvalidType('Cannot insert to db. Expected InternalScope, got string type.')
         else:
             return value.internal
