@@ -1,47 +1,43 @@
-'''
-  Copyright European Organization for Nuclear Research (CERN)
+# -*- coding: utf-8 -*-
+# Copyright European Organization for Nuclear Research (CERN) since 2012
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
-  Licensed under the Apache License, Version 2.0 (the "License");
-  You may not use this file except in compliance with the License.
-  You may obtain a copy of the License at
-  http://www.apache.org/licenses/LICENSE-2.0
-
-  Authors:
-  - Cedric Serfon <cedric.serfon@cern.ch>, 2012-2017
-  - Vincent Garonne, <vincent.garonne@cern.ch>, 2013-2017
-  - Sylvain Blunier, <sylvain.blunier@cern.ch>, 2016
-  - Nicolo Magini, <nicolo.magini@cern.ch>, 2018
-  - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2019
-
-  PY3K COMPATIBLE
-'''
-
-from __future__ import print_function, division
 import os
-import ssl
 import sys
 
 import xml.etree.ElementTree as ET
 from xml.parsers import expat
 
 import requests
-from progressbar import ProgressBar
 from requests.adapters import HTTPAdapter
 from urllib3.poolmanager import PoolManager
+from urllib.parse import urlparse
 
 from rucio.common import exception
 from rucio.rse.protocols import protocol
 
 
-class TLSv1HttpAdapter(HTTPAdapter):
+class TLSHTTPAdapter(HTTPAdapter):
     '''
-    Class to force the SSL protocol to TLSv1
+    Class to force the SSL protocol to latest TLS
     '''
     def init_poolmanager(self, connections, maxsize, block=False):
         self.poolmanager = PoolManager(num_pools=connections,
                                        maxsize=maxsize,
                                        block=block,
-                                       ssl_version=ssl.PROTOCOL_TLSv1)
+                                       cert_reqs="CERT_REQUIRED",
+                                       ca_cert_dir="/etc/grid-security/certificates")
 
 
 class UploadInChunks(object):
@@ -152,14 +148,15 @@ class Default(protocol.RSEProtocol):
     def connect(self, credentials={}):
         """ Establishes the actual connection to the referred RSE.
 
-            :param credentials Provides information to establish a connection
+            :param credentials: Provides information to establish a connection
                 to the referred storage system. For WebDAV connections these are
                 ca_cert, cert, auth_type, timeout
 
             :raises RSEAccessDenied
         """
         try:
-            self.server = self.path2pfn('')
+            parse_url = urlparse(self.path2pfn(''))
+            self.server = f'{parse_url.scheme}://{parse_url.netloc}'
         except KeyError:
             raise exception.RSEAccessDenied('No specified Server')
 
@@ -193,7 +190,7 @@ class Default(protocol.RSEProtocol):
         except KeyError:
             self.timeout = 300
         self.session = requests.Session()
-        self.session.mount('https://', TLSv1HttpAdapter())
+        self.session.mount('https://', TLSHTTPAdapter())
         if self.auth_token:
             self.session.headers.update({'Authorization': 'Bearer ' + self.auth_token})
         # "ping" to see if the server is available
@@ -226,7 +223,7 @@ class Default(protocol.RSEProtocol):
     def exists(self, pfn):
         """ Checks if the requested file is known by the referred RSE.
 
-            :param pfn Physical file name
+            :param pfn: Physical file name
 
             :returns: True if the file exists, False if it doesn't
 
@@ -250,8 +247,8 @@ class Default(protocol.RSEProtocol):
     def get(self, pfn, dest='.', transfer_timeout=None):
         """ Provides access to files stored inside connected the RSE.
 
-            :param pfn Physical file name of requested file
-            :param dest Name and path of the files when stored at the client
+            :param pfn: Physical file name of requested file
+            :param dest: Name and path of the files when stored at the client
             :param transfer_timeout: Transfer timeout (in seconds) - dummy
 
             :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound, RSEAccessDenied
@@ -264,23 +261,14 @@ class Default(protocol.RSEProtocol):
                 length = None
                 if 'content-length' in result.headers:
                     length = int(result.headers['content-length'])
-                    totnchunk = int(length / chunksize) + 1
                 with open(dest, 'wb') as file_out:
                     nchunk = 0
-                    try:
+                    if not length:
+                        print('Malformed HTTP response (missing content-length header).')
+                    for chunk in result.iter_content(chunksize):
+                        file_out.write(chunk)
                         if length:
-                            pbar = ProgressBar(maxval=totnchunk).start()
-                        else:
-                            print('Malformed HTTP response (missing content-length header). Cannot show progress bar.')
-                        for chunk in result.iter_content(chunksize):
-                            file_out.write(chunk)
-                            if length:
-                                nchunk += 1
-                                pbar.update(nchunk)
-                    finally:
-                        if length:
-                            pbar.finish()
-
+                            nchunk += 1
             elif result.status_code in [404, ]:
                 raise exception.SourceNotFound()
             elif result.status_code in [401, 403]:
@@ -296,8 +284,8 @@ class Default(protocol.RSEProtocol):
     def put(self, source, target, source_dir=None, transfer_timeout=None, progressbar=False):
         """ Allows to store files inside the referred RSE.
 
-            :param source Physical file name
-            :param target Name of the file on the storage system e.g. with prefixed scope
+            :param source: Physical file name
+            :param target: Name of the file on the storage system e.g. with prefixed scope
             :param source_dir Path where the to be transferred files are stored in the local file system
             :param transfer_timeout Transfer timeout (in seconds) - dummy
 
@@ -349,7 +337,7 @@ class Default(protocol.RSEProtocol):
     def rename(self, pfn, new_pfn):
         """ Allows to rename a file stored inside the connected RSE.
 
-            :param pfn      Current physical file name
+            :param pfn:      Current physical file name
             :param new_pfn  New physical file name
 
             :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound, RSEAccessDenied
@@ -392,7 +380,7 @@ class Default(protocol.RSEProtocol):
     def delete(self, pfn):
         """ Deletes a file from the connected RSE.
 
-            :param pfn Physical file name
+            :param pfn: Physical file name
 
             :raises ServiceUnavailable, SourceNotFound, RSEAccessDenied, ResourceTemporaryUnavailable
         """
@@ -418,7 +406,7 @@ class Default(protocol.RSEProtocol):
     def mkdir(self, directory):
         """ Internal method to create directories
 
-            :param directory Name of the directory that needs to be created
+            :param directory: Name of the directory that needs to be created
 
             :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound, RSEAccessDenied
         """
@@ -442,7 +430,7 @@ class Default(protocol.RSEProtocol):
     def ls(self, filename):
         """ Internal method to list files/directories
 
-            :param filename Name of the directory that needs to be created
+            :param filename: Name of the directory that needs to be created
 
             :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound, RSEAccessDenied
         """
@@ -485,9 +473,8 @@ class Default(protocol.RSEProtocol):
 
             :returns: a dict with two keys, filesize and adler32 of the file provided in path.
         """
-        raise NotImplementedError
         headers = {'Depth': '1'}
-        dict = {}
+        dict_ = {}
         try:
             result = self.session.request('PROPFIND', path, verify=False, headers=headers, timeout=self.timeout, cert=self.cert)
             if result.status_code in [404, ]:
@@ -495,14 +482,14 @@ class Default(protocol.RSEProtocol):
             elif result.status_code in [401, ]:
                 raise exception.RSEAccessDenied()
             if result.status_code in [400, ]:
-                raise NotImplementedError
+                raise exception.InvalidRequest()
             parser = Parser()
             parser.feed(result.text)
             for file_name in parser.sizes:
                 if '%s%s' % (self.server, file_name) == path:
-                    dict['size'] = parser.sizes[file_name]
+                    dict_['filesize'] = parser.sizes[file_name]
             parser.close()
-            return dict
+            return dict_
         except requests.exceptions.ConnectionError as error:
             raise exception.ServiceUnavailable(error)
         except requests.exceptions.ReadTimeout as error:
@@ -524,7 +511,7 @@ class Default(protocol.RSEProtocol):
             usedsize = root[0][1][0].find('{DAV:}quota-used-bytes').text
             try:
                 unusedsize = root[0][1][0].find('{DAV:}quota-available-bytes').text
-            except Exception as error:
+            except Exception:
                 print('No free space given, return -999')
                 unusedsize = -999
             totalsize = int(usedsize) + int(unusedsize)
