@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,17 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2018-2020
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2019
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
-# - Martin Barisits <martin.barisits@cern.ch>, 2019
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2022
 
 import unittest
 
@@ -60,7 +49,7 @@ class TestCredential(unittest.TestCase):
                                     'impl': 'rucio.rse.protocols.gfal.Default',
                                     'domains': {
                                         'lan': {'read': 1, 'write': 1, 'delete': 1},
-                                        'wan': {'read': 1, 'write': 1, 'delete': 1, 'third_party_copy': 1}}})
+                                        'wan': {'read': 1, 'write': 1, 'delete': 1, 'third_party_copy_read': 1, 'third_party_copy_write': 1}}})
 
         add_protocol(self.rse2_id, {'scheme': 'https',
                                     'hostname': 'storage.googleapis.com',
@@ -69,7 +58,7 @@ class TestCredential(unittest.TestCase):
                                     'impl': 'rucio.rse.protocols.gfal.Default',
                                     'domains': {
                                         'lan': {'read': 1, 'write': 1, 'delete': 1},
-                                        'wan': {'read': 1, 'write': 1, 'delete': 1, 'third_party_copy': 1}}})
+                                        'wan': {'read': 1, 'write': 1, 'delete': 1, 'third_party_copy_read': 1, 'third_party_copy_write': 1}}})
 
         # register some files there
         self.files = [{'scope': InternalScope('mock', **self.vo),
