@@ -25,18 +25,27 @@ from rucio.common.exception import (Duplicate, RSENotFound, RSEProtocolNotSuppor
                                     InvalidObject, ResourceTemporaryUnavailable,
                                     RSEAttributeNotFound, RSEOperationNotSupported,
                                     InputValidationError)
-from rucio.common.utils import generate_uuid
+from rucio.common.schema import get_schema_value
+from rucio.common.types import InternalAccount, InternalScope
+from rucio.common.utils import GLOBALLY_SUPPORTED_CHECKSUMS, CHECKSUM_KEY
+from rucio.core.account_limit import set_local_account_limit
+from rucio.core.did import add_did, attach_dids
+from rucio.core.rule import add_rule
 from rucio.core.rse import (add_rse, get_rse_id, del_rse, restore_rse, list_rses,
                             rse_exists, add_rse_attribute, list_rse_attributes,
                             set_rse_transfer_limits, get_rse_transfer_limits,
                             delete_rse_transfer_limits, get_rse_protocols,
                             del_rse_attribute, get_rse_attribute, get_rse, rse_is_empty,
+                            parse_checksum_support_attribute,
+                            get_rse_supported_checksums_from_attributes,
                             update_rse)
+from rucio.daemons.abacus.account import account_update
 from rucio.db.sqla import session, models
-from rucio.db.sqla.constants import RSEType
+from rucio.db.sqla.constants import RSEType, DIDType
 from rucio.rse import rsemanager as mgr
-from rucio.tests.common import rse_name_generator, hdrdict, auth, headers
+from rucio.tests.common import rse_name_generator, hdrdict, auth, headers, did_name_generator
 from rucio.tests.common_server import get_vo
+from rucio.tests.test_rule import create_files
 
 
 class TestRSECoreApi(unittest.TestCase):
@@ -155,7 +164,7 @@ class TestRSECoreApi(unittest.TestCase):
         rse_name = rse_name_generator()
         rse_id = add_rse(rse_name, **self.vo)
         del_rse_attribute(rse_id=rse_id, key=rse_name)
-        assert get_rse_attribute(key=rse_name, rse_id=rse_id) == []
+        assert get_rse_attribute(rse_id, rse_name) is None
 
         with pytest.raises(RSEAttributeNotFound):
             del_rse_attribute(rse_id=rse_id, key=rse_name)
@@ -217,6 +226,40 @@ class TestRSECoreApi(unittest.TestCase):
         assert not rse_is_empty(rse_id=rse_id)
 
 
+def test_parse_checksum_support_attribute():
+    assert parse_checksum_support_attribute('') == GLOBALLY_SUPPORTED_CHECKSUMS
+    assert parse_checksum_support_attribute('none') == []
+    assert parse_checksum_support_attribute('none,md5') == []
+    assert parse_checksum_support_attribute('md5') == ['md5']
+    assert parse_checksum_support_attribute('md5,adler32') == ['md5', 'adler32']
+
+
+@pytest.mark.parametrize("caches_mock", [{
+    "caches_to_mock": ['rucio.core.rse.REGION'],
+    'expiration_time': 0
+}], indirect=True)
+def test_rse_get_supported_checksums_from_attributes(vo, caches_mock):
+    rse_name = rse_name_generator()
+    rse_id = add_rse(rse_name, vo=vo)
+
+    attrs = list_rse_attributes(rse_id)
+    assert get_rse_supported_checksums_from_attributes(attrs) == GLOBALLY_SUPPORTED_CHECKSUMS
+
+    add_rse_attribute(rse_id, CHECKSUM_KEY, 'none')
+    attrs = list_rse_attributes(rse_id)
+    assert get_rse_supported_checksums_from_attributes(attrs) == []
+
+    add_rse_attribute(rse_id, CHECKSUM_KEY, 'md5')
+    attrs = list_rse_attributes(rse_id)
+    assert get_rse_supported_checksums_from_attributes(attrs) == ['md5']
+
+    add_rse_attribute(rse_id, CHECKSUM_KEY, 'md5,adler32')
+    attrs = list_rse_attributes(rse_id)
+    assert get_rse_supported_checksums_from_attributes(attrs) == ['md5', 'adler32']
+
+    del_rse(rse_id)
+
+
 def test_create_rse_success(vo, rest_client, auth_token):
     """ RSE (REST): send a POST to create a new RSE """
     rse_name = rse_name_generator()
@@ -405,9 +448,9 @@ class TestRSEClient(unittest.TestCase):
         # Check if updating RSE does not remove RSE tag
         rse = rse_name_generator()
         ret = self.client.add_rse(rse)
-        assert get_rse_attribute(key=rse, rse_id=get_rse_id(rse, **self.vo)) == [True]
+        assert get_rse_attribute(get_rse_id(rse, **self.vo), rse) is True
         self.client.update_rse(rse, {'availability_write': False, 'availability_delete': False})
-        assert get_rse_attribute(key=rse, rse_id=get_rse_id(rse, **self.vo)) == [True]
+        assert get_rse_attribute(get_rse_id(rse, **self.vo), rse) is True
 
         rse = rse_name_generator()
         renamed_rse = 'renamed_rse%s' % rse
@@ -421,7 +464,7 @@ class TestRSEClient(unittest.TestCase):
 
         tmp_scope = 'mock'
         nbfiles = 5
-        files1 = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1,
+        files1 = [{'scope': tmp_scope, 'name': did_name_generator('file'), 'bytes': 1,
                    'adler32': '0cc737eb', 'meta': {'events': 10}} for i in range(nbfiles)]
         replica_client = ReplicaClient()
         replica_client.add_replicas(rse=renamed_rse, files=files1)
@@ -432,7 +475,7 @@ class TestRSEClient(unittest.TestCase):
         assert dict2['availability_write'] is False
         assert dict2['availability_delete'] is False
 
-        files2 = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1,
+        files2 = [{'scope': tmp_scope, 'name': did_name_generator('file'), 'bytes': 1,
                    'adler32': '0cc737eb', 'meta': {'events': 10}} for i in range(nbfiles)]
         with pytest.raises(ResourceTemporaryUnavailable):
             replica_client.add_replicas(rse=renamed_rse, files=files2, ignore_availability=False)
@@ -1327,10 +1370,24 @@ class TestRSEClient(unittest.TestCase):
 
     def test_get_rse_usage(self):
         """ RSE (CLIENTS): Test getting the RSE usage. """
-        usages = self.client.get_rse_usage(rse='MOCK', filters={'per_account': True})
+        file_sizes = 100
+        nfiles = 3
+        jdoe_account = InternalAccount('jdoe', **self.vo)
+        rse, rse_id = self.rse_factory.make_posix_rse()
+        set_local_account_limit(account=jdoe_account, rse_id=rse_id, bytes_=10000)
+        tmp_scope = InternalScope('mock', **self.vo)
+        activity = get_schema_value('ACTIVITY')['enum'][0]
+        files = create_files(nfiles, tmp_scope, rse_id, bytes_=file_sizes)
+        dataset = did_name_generator('dataset')
+        add_did(tmp_scope, dataset, DIDType.DATASET, jdoe_account)
+        attach_dids(tmp_scope, dataset, files, jdoe_account)
+        rules = add_rule(dids=[{'scope': tmp_scope, 'name': dataset}], account=jdoe_account, copies=1, rse_expression=rse, grouping='DATASET', weight=None, lifetime=None, locked=False, subscription_id=None, activity=activity)
+        assert rules
+        account_update(once=True)
+        usages = self.client.get_rse_usage(rse=rse, filters={'per_account': True})
         for usage in usages:
             assert usage['account_usages']
-        usages = self.client.get_rse_usage(rse='MOCK')
+        usages = self.client.get_rse_usage(rse=rse)
         for usage in usages:
             assert 'account_usages' not in usage
 
@@ -1446,7 +1503,7 @@ class TestRSEClient(unittest.TestCase):
         rse_name = rse_name_generator()
         self.client.add_rse(rse_name)
         self.client.delete_rse_attribute(rse=rse_name, key=rse_name)
-        assert get_rse_attribute(key=rse_name, rse_id=get_rse_id(rse_name, **self.vo)) == []
+        assert get_rse_attribute(get_rse_id(rse_name, **self.vo), rse_name) is None
 
         with pytest.raises(RSEAttributeNotFound):
             self.client.delete_rse_attribute(rse=rse_name, key=rse_name)
@@ -1484,3 +1541,25 @@ class TestRSEClient(unittest.TestCase):
         with pytest.raises(InputValidationError):
             update_rse(rse_id, parameters={'city': 'Not Berlin', 'non_existing_option': 3})
         assert get_rse(rse_id)['city'] == 'Berlin'
+
+
+@pytest.mark.parametrize("use_cache", [
+    False,
+    pytest.param(True, marks=pytest.mark.xfail(reason='FIXME: Calling functions which change the rse attribute should invalidate the cache.')),
+])
+def test_get_rse_attribute(use_cache, rse_factory):
+    _, rse_id = rse_factory.make_mock_rse()
+
+    assert get_rse_attribute(rse_id, "test") is None
+
+    add_rse_attribute(rse_id, "test", "test")
+    assert get_rse_attribute(rse_id, "test", use_cache=use_cache) == "test"
+
+    add_rse_attribute(rse_id, "test", True)
+    assert get_rse_attribute(rse_id, "test", use_cache=use_cache) is True
+
+    add_rse_attribute(rse_id, "test", False)
+    assert get_rse_attribute(rse_id, "test", use_cache=use_cache) is False
+
+    del_rse_attribute(rse_id, "test")
+    assert get_rse_attribute(rse_id, "test", use_cache=use_cache) is None
