@@ -1,4 +1,5 @@
-# Copyright 2013-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,62 +12,79 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - WeiJen Chang <e4523744@gmail.com>, 2013
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2013
-# - Cheng-Hsi Chao <cheng-hsi.chao@cern.ch>, 2014
-# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2018
-# - Vincent Garonne <vgaronne@gmail.com>, 2018
 
-from __future__ import print_function
-
-import json
 import os
 import shutil
 import tempfile
-
+import unittest
 from uuid import uuid4 as uuid
 
-from nose.tools import raises
+import pytest
 
 from rucio.common import exception
-from rucio.rse import rsemanager
-from rsemgr_api_test import MgrTestCases
 from rucio.common.utils import execute
+from rucio.rse import rsemanager
+from rucio.tests.common import skip_rse_tests_with_accounts, load_test_conf_file
+from rucio.tests.rsemgr_api_test import MgrTestCases
 
 
-class TestRseXROOTD():
+@pytest.mark.noparallel(reason='creates and removes a test directory with a fixed name')
+@skip_rse_tests_with_accounts
+class TestRseXROOTD(unittest.TestCase):
     tmpdir = None
     user = None
 
     @classmethod
-    def setupClass(cls):
+    def get_rse_info(cls):
+        """
+        Detects if containerized rses for xrootd are available in the testing environment.
+        :return: A tuple (rse, prefix, hostname, port).
+        """
+        cmd = "rucio list-rses --rses 'test_container_xrd=True'"
+        print(cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        rses = out.split()
+
+        data = load_test_conf_file('rse_repository.json')
+        prefix = data['WJ-XROOTD']['protocols']['supported']['xroot']['prefix']
+        port = data['WJ-XROOTD']['protocols']['supported']['xroot']['port']
+
+        if len(rses) == 0:
+            rse_id = 'WJ-XROOTD'
+            hostname = data['WJ-XROOTD']['protocols']['supported']['xroot']['hostname']
+        else:
+            rse_id = 'XRD1'
+            hostname = 'xrd1'
+            prefix = '/rucio/'
+        # TODO should read container info from a config file
+        return rse_id, prefix, hostname, port
+
+    @classmethod
+    def setUpClass(cls):
         """XROOTD (RSE/PROTOCOLS): Creating necessary directories and files """
+
+        # Getting info for the test environment
+        rse_id, prefix, hostname, port = cls.get_rse_info()
+
+        try:
+            os.mkdir(prefix)
+        except Exception as e:
+            print(e)
+
         # Creating local files
         cls.tmpdir = tempfile.mkdtemp()
         cls.user = uuid()
 
         with open("%s/data.raw" % cls.tmpdir, "wb") as out:
             out.seek((1024 * 1024) - 1)  # 1 MB
-            out.write('\0')
+            out.write(b'\0')
         for f in MgrTestCases.files_local:
             shutil.copy('%s/data.raw' % cls.tmpdir, '%s/%s' % (cls.tmpdir, f))
 
-        protocol = rsemanager.create_protocol(rsemanager.get_rse_info('WJ-XROOTD'), 'write')
+        protocol = rsemanager.create_protocol(rsemanager.get_rse_info(rse_id), 'write')
         protocol.connect()
 
-        with open('etc/rse_repository.json') as f:
-            data = json.load(f)
-        prefix = data['WJ-XROOTD']['protocols']['supported']['xroot']['prefix']
-        hostname = data['WJ-XROOTD']['protocols']['supported']['xroot']['hostname']
-        port = data['WJ-XROOTD']['protocols']['supported']['xroot']['port']
-
-        try:
-            os.mkdir(prefix)
-        except Exception as e:
-            print(e)
-
         os.system('dd if=/dev/urandom of=%s/data.raw bs=1024 count=1024' % prefix)
         cls.static_file = 'xroot://%s:%d/%s/data.raw' % (hostname, port, prefix)
         cmd = 'xrdcp %s/data.raw %s' % (prefix, cls.static_file)
@@ -77,13 +95,16 @@ class TestRseXROOTD():
             cmd = 'xrdcp %s/data.raw %s' % (prefix, path)
             execute(cmd)
 
+        for f in MgrTestCases.files_local_and_remote:
+            shutil.copy('%s/data.raw' % cls.tmpdir, '%s/%s' % (cls.tmpdir, f))
+            path = protocol.path2pfn(prefix + protocol._get_path('user.%s' % cls.user, f))
+            cmd = 'xrdcp %s/%s %s' % (cls.tmpdir, f, path)
+            execute(cmd)
+
     @classmethod
-    def teardownClass(cls):
+    def tearDownClass(cls):
         """XROOTD (RSE/PROTOCOLS): Removing created directorie s and files"""
-        with open('etc/rse_repository.json') as f:
-            data = json.load(f)
-        prefix = data['WJ-XROOTD']['protocols']['supported']['xroot']['prefix']
-        hostname = data['WJ-XROOTD']['protocols']['supported']['xroot']['hostname']
+        rse_id, prefix, hostname, port = cls.get_rse_info()
 
         shutil.rmtree(prefix)
         shutil.rmtree(cls.tmpdir)
@@ -106,39 +127,11 @@ class TestRseXROOTD():
             clean_cmd = 'xrdfs %s rmdir %s' % (hostname, directory)
             execute(clean_cmd)
 
-    def setup(self):
+    def setUp(self):
         """XROOTD (RSE/PROTOCOLS): Creating Mgr-instance """
         self.tmpdir = TestRseXROOTD.tmpdir
-        self.rse_id = 'WJ-XROOTD'
-        self.mtc = MgrTestCases(self.tmpdir, 'WJ-XROOTD', TestRseXROOTD.user, TestRseXROOTD.static_file)
-
-    # Mgr-Tests: GET
-    def test_multi_get_mgr_ok(self):
-        """XROOTD (RSE/PROTOCOLS): Get multiple files from storage providing LFNs and PFNs (Success)"""
-        self.mtc.test_multi_get_mgr_ok()
-
-    def test_get_mgr_ok_single_lfn(self):
-        """XROOTD (RSE/PROTOCOLS): Get a single file from storage providing LFN (Success)"""
-        self.mtc.test_get_mgr_ok_single_lfn()
-
-    def test_get_mgr_ok_single_pfn(self):
-        """XROOTD (RSE/PROTOCOLS): Get a single file from storage providing PFN (Success)"""
-        self.mtc.test_get_mgr_ok_single_pfn()
-
-    @raises(exception.SourceNotFound)
-    def test_get_mgr_SourceNotFound_multi(self):
-        """XROOTD (RSE/PROTOCOLS): Get multiple files from storage providing LFNs and PFNs (SourceNotFound)"""
-        self.mtc.test_get_mgr_SourceNotFound_multi()
-
-    @raises(exception.SourceNotFound)
-    def test_get_mgr_SourceNotFound_single_lfn(self):
-        """XROOTD (RSE/PROTOCOLS): Get a single file from storage providing LFN (SourceNotFound)"""
-        self.mtc.test_get_mgr_SourceNotFound_single_lfn()
-
-    @raises(exception.SourceNotFound)
-    def test_get_mgr_SourceNotFound_single_pfn(self):
-        """XROOTD (RSE/PROTOCOLS): Get a single file from storage providing PFN (SourceNotFound)"""
-        self.mtc.test_get_mgr_SourceNotFound_single_pfn()
+        self.rse_id, self.prefix, self.hostname, self.port = TestRseXROOTD.get_rse_info()
+        self.mtc = MgrTestCases(self.tmpdir, self.rse_id, TestRseXROOTD.user, TestRseXROOTD.static_file)
 
     # Mgr-Tests: PUT
     def test_put_mgr_ok_multi(self):
@@ -149,25 +142,25 @@ class TestRseXROOTD():
         """XROOTD (RSE/PROTOCOLS): Put a single file to storage (Success)"""
         self.mtc.test_put_mgr_ok_single()
 
-    @raises(exception.SourceNotFound)
     def test_put_mgr_SourceNotFound_multi(self):
         """XROOTD (RSE/PROTOCOLS): Put multiple files to storage (SourceNotFound)"""
-        self.mtc.test_put_mgr_SourceNotFound_multi()
+        with pytest.raises(exception.SourceNotFound):
+            self.mtc.test_put_mgr_SourceNotFound_multi()
 
-    @raises(exception.SourceNotFound)
     def test_put_mgr_SourceNotFound_single(self):
         """XROOTD (RSE/PROTOCOLS): Put a single file to storage (SourceNotFound)"""
-        self.mtc.test_put_mgr_SourceNotFound_single()
+        with pytest.raises(exception.SourceNotFound):
+            self.mtc.test_put_mgr_SourceNotFound_single()
 
-    @raises(exception.FileReplicaAlreadyExists)
     def test_put_mgr_FileReplicaAlreadyExists_multi(self):
         """XROOTD (RSE/PROTOCOLS): Put multiple files to storage (FileReplicaAlreadyExists)"""
-        self.mtc.test_put_mgr_FileReplicaAlreadyExists_multi()
+        with pytest.raises(exception.FileReplicaAlreadyExists):
+            self.mtc.test_put_mgr_FileReplicaAlreadyExists_multi()
 
-    @raises(exception.FileReplicaAlreadyExists)
     def test_put_mgr_FileReplicaAlreadyExists_single(self):
         """XROOTD (RSE/PROTOCOLS): Put a single file to storage (FileReplicaAlreadyExists)"""
-        self.mtc.test_put_mgr_FileReplicaAlreadyExists_single()
+        with pytest.raises(exception.FileReplicaAlreadyExists):
+            self.mtc.test_put_mgr_FileReplicaAlreadyExists_single()
 
     # MGR-Tests: DELETE
     def test_delete_mgr_ok_multi(self):
@@ -178,15 +171,15 @@ class TestRseXROOTD():
         """XROOTD (RSE/PROTOCOLS): Delete a single file from storage (Success)"""
         self.mtc.test_delete_mgr_ok_single()
 
-    @raises(exception.SourceNotFound)
     def test_delete_mgr_SourceNotFound_multi(self):
         """XROOTD (RSE/PROTOCOLS): Delete multiple files from storage (SourceNotFound)"""
-        self.mtc.test_delete_mgr_SourceNotFound_multi()
+        with pytest.raises(exception.SourceNotFound):
+            self.mtc.test_delete_mgr_SourceNotFound_multi()
 
-    @raises(exception.SourceNotFound)
     def test_delete_mgr_SourceNotFound_single(self):
         """XROOTD (RSE/PROTOCOLS): Delete a single file from storage (SourceNotFound)"""
-        self.mtc.test_delete_mgr_SourceNotFound_single()
+        with pytest.raises(exception.SourceNotFound):
+            self.mtc.test_delete_mgr_SourceNotFound_single()
 
     # MGR-Tests: EXISTS
     def test_exists_mgr_ok_multi(self):
@@ -226,35 +219,35 @@ class TestRseXROOTD():
         """XROOTD (RSE/PROTOCOLS): Rename a single file on storage using PFN (Success)"""
         self.mtc.test_rename_mgr_ok_single_pfn()
 
-    @raises(exception.FileReplicaAlreadyExists)
     def test_rename_mgr_FileReplicaAlreadyExists_multi(self):
         """XROOTD (RSE/PROTOCOLS): Rename multiple files on storage (FileReplicaAlreadyExists)"""
-        self.mtc.test_rename_mgr_FileReplicaAlreadyExists_multi()
+        with pytest.raises(exception.FileReplicaAlreadyExists):
+            self.mtc.test_rename_mgr_FileReplicaAlreadyExists_multi()
 
-    @raises(exception.FileReplicaAlreadyExists)
     def test_rename_mgr_FileReplicaAlreadyExists_single_lfn(self):
         """XROOTD (RSE/PROTOCOLS): Rename a single file on storage using LFN(FileReplicaAlreadyExists)"""
-        self.mtc.test_rename_mgr_FileReplicaAlreadyExists_single_lfn()
+        with pytest.raises(exception.FileReplicaAlreadyExists):
+            self.mtc.test_rename_mgr_FileReplicaAlreadyExists_single_lfn()
 
-    @raises(exception.FileReplicaAlreadyExists)
     def test_rename_mgr_FileReplicaAlreadyExists_single_pfn(self):
         """XROOTD (RSE/PROTOCOLS): Rename a single file on storage using PFN (FileReplicaAlreadyExists)"""
-        self.mtc.test_rename_mgr_FileReplicaAlreadyExists_single_pfn()
+        with pytest.raises(exception.FileReplicaAlreadyExists):
+            self.mtc.test_rename_mgr_FileReplicaAlreadyExists_single_pfn()
 
-    @raises(exception.SourceNotFound)
     def test_rename_mgr_SourceNotFound_multi(self):
         """XROOTD (RSE/PROTOCOLS): Rename multiple files on storage (SourceNotFound)"""
-        self.mtc.test_rename_mgr_SourceNotFound_multi()
+        with pytest.raises(exception.SourceNotFound):
+            self.mtc.test_rename_mgr_SourceNotFound_multi()
 
-    @raises(exception.SourceNotFound)
     def test_rename_mgr_SourceNotFound_single_lfn(self):
         """XROOTD (RSE/PROTOCOLS): Rename a single file on storage using LFN (SourceNotFound)"""
-        self.mtc.test_rename_mgr_SourceNotFound_single_lfn()
+        with pytest.raises(exception.SourceNotFound):
+            self.mtc.test_rename_mgr_SourceNotFound_single_lfn()
 
-    @raises(exception.SourceNotFound)
     def test_rename_mgr_SourceNotFound_single_pfn(self):
         """XROOTD (RSE/PROTOCOLS): Rename a single file on storage using PFN (SourceNotFound)"""
-        self.mtc.test_rename_mgr_SourceNotFound_single_pfn()
+        with pytest.raises(exception.SourceNotFound):
+            self.mtc.test_rename_mgr_SourceNotFound_single_pfn()
 
     def test_change_scope_mgr_ok_single_lfn(self):
         """XROOTD (RSE/PROTOCOLS): Change the scope of a single file on storage using LFN (Success)"""
