@@ -573,7 +573,8 @@ class FTS3ApiTransferStatusReport(Fts3TransferStatusReport):
         if file_state_is_final:
             if file_state == FTS_STATE.FINISHED:
                 new_state = RequestState.DONE
-            elif file_state == FTS_STATE.FAILED and not self._multi_sources:  # for multi-source transfers we must wait for the job to be in a final state
+            elif file_state == FTS_STATE.FAILED and job_state == FTS_STATE.FAILED or \
+                    file_state == FTS_STATE.FAILED and not self._multi_sources:  # for multi-source transfers we must wait for the job to be in a final state
                 if self._is_recoverable_fts_overwrite_error(self.request(session), reason, self._file_metadata):
                     new_state = RequestState.DONE
                 else:
@@ -726,6 +727,7 @@ class FTS3Transfertool(Transfertool):
             bring_online=self.bring_online,
             default_lifetime=self.default_lifetime,
             archive_timeout_override=self.archive_timeout_override,
+            logger=self.logger,
         )
         return jobs
 
@@ -1184,7 +1186,7 @@ class FTS3Transfertool(Transfertool):
         """
         result = (None, None)
         try:
-            key = 'voname: %s' % self.external_host
+            key = 'voname:%s' % self.external_host
             result = REGION_SHORT.get(key)
             if isinstance(result, NoValue):
                 self.logger(logging.DEBUG, "Refresh transfer baseid and voname for %s", self.external_host)
